#define NRANSI
#include <time.h> 
#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

/*************************************/
/*** INTEGRATION                   ***/
/*************************************/
#define RESTART 0           /* restart flag, 1= read initial system state */
#define TMAX 200.0         /* number of years to simulated */
#define RPT_INTEGRATION 1  /* reintegration using euler step if necessary */
#define N_CLIMATE 12       /* num of climate values (12 = month */
#define N_SUB 12           /* num of substeps in year */
#define N_OUT 10           /* num of substeps in ode integrator, !SODE only*/ 
#define N_GSTEPS 1         /* numb of growth substeps, !SODE only */
#define TIMESTEP 1.0/N_SUB /* integration timestep (yrs)*/ 
#define PREFIX "pdh"
#define PATCH_DYNAMICS 1   /* patch dynamics flag, 1=yes to patch dynamics */ 
#define PATCH_FREQ N_SUB   /* freq of patch_dynamics in N_SUB units (yrly)*/
#define N_SUB_PER_PATCH_FREQ N_SUB/PATCH_FREQ
#define COHORT_FREQ 1      /* freq of cohort_dynamics in N_SUB units (mnthly)*/
#define RARIFY_FACTOR 1    
/**************************************/
/*** REGION                         ***/
/**************************************/
#define REGION 0 /* 0= run at sois only, print soi files only        */ 
                 /* 1 =run over region, print regional and soi files */
#define LATMIN  -15  /* integer coordinates for region in degrees */
#define LATMAX  15  /* S:-15 N:15 W:-85 E:-30 */  
#define LONMIN -85  
#define LONMAX -30 

/**************************************/
/*** BIOLOGY/BIOGEOCHEMISTRY        ***/
/**************************************/
#define WATER_COMPETITION 1 /* competition for water flag, 1=yes */
#define N_COMPETITION 0     /* competition for N flag, 1=yes*/
#define N_DECOMP_LIMITATION 1 /* N control of structuralC decomp, 1=yes */ 
#define INTERNAL_RECRUITMENT 1 /* internal recruitment flag 1=yes */ 
#define EXTERNAL_RECRUITMENT 0 /* external fixed recruitment flag 1=yes */ 
#define N_DIST_TYPES 2  /*number of disturbance dist_types*/

#define N_PFT 4       /* number of plant functional types */
#define PT 2          /* number of different plant physiologies */
#define NCMPT 6       /* number carbon compartments in allocation array */
#define HGTMIN 0.5    /* lowest height value assigned to data->hgt_min */
#define C2B 2.0       /* carbon to biomass conversion */
#define GROWTH_RESP 0.333 /* fraction of npp lost as growth respiration */
#define MASS_OF_WATER 1.0 /*(kg/mm/m2) */

/**************************************/
/*** FUSION/FISSION                 ***/
/**************************************/
#define PATCH_TERMINATION 1  /* terminate small patches and adj area         */
#define PATCH_FUSION 1       /* patch fusion flag,   1=yes to patch fusion   */
#define COHORT_FUSION 1      /* cohort fusion flag,  1=yes to cohort fusion  */
#define COHORT_FISSION 1     /* cohort fission flag, 1=yes to cohort fission */
#define COHORT_TERMINATION 1 /* terminate small cohorts                      */

#define F_AREA  0.01    /* min area of patch as a fraction of total area */
#define BTOL 0.00001    /* termination tol. for cohort biomass (kg m^-2)  */
#define PROFILE_TOL 1.0 /* 0.2 fractional tol. for patch pft hgt profiles */
#define DBHMAX 200.0     /* max dbh value used in hgt profile comparison */
#define N_DBH_BINS 20    /* no. of dbh bins used when comparing patches */
#define NTOL 0.001       /* min plant density for hgt bin to be used in height profile comparisons */
#define FUSETOL 0.4       /* min fractional difference in dbh bet cohorts*/
#define LAI_TOL 1.0       /* maximum LAI allowed for a cohort */

/**************************************/
/*** MISC                           ***/
/**************************************/
#define COHORT_SHADING 0.5   /* degree of within cohort shading */ 
#define SELF_SHADING 0.0     /* degree of self shading */   
#define CELL_AREA 225.0
#define N_INIT_PATCHES 1   
#define AREA 10000.0 /*set in pde to reasonable value, say 10000.0, for numerics, actual site area is read in, is huge, and can be used for area dependent calcs later*/ 
#define DBH_SORT 1           /* flag to sort on dbh rather than height */
#define LIGHT_RESOLUTION 20  /* resolution of light in lut */
#define MIN_LIGHT 6
#define N_LIGHT LIGHT_RESOLUTION*MIN_LIGHT+1 /* number light bins in lut */

#define INIT_DENSITY 0.1 /* 0.1 Initial plant density (indivs/m2) */
#define RESOLUTION 1.0   /* grid cell size in degress lat/long */

#define ABS(a)  ( (a)<0 ? (-a) : (a))
#define FLR(a)((a) > 0.0 ? a : 0.0)
#define STR_LEN 512
/**************************************/
/*** DEBUGGING                      ***/
/**************************************/
#define DEBUG_INSERT_COHORT 0
#define DEBUG_CREATE_COHORT 0
#define DEBUG_SPAWN_COHORTS 0
#define DEBUG_SPLIT_COHORTS 0
#define DEBUG_FUSE_COHORTS 0
#define DEBUG_SPAWN_PATCHES 0
#define DEBUG_FUSE_PATCHES 0
#define DEBUG_LIGHT 0 
#define DEBUG_PHENOLOGY 0
#define DEBUG_LITTER 0
#define DEBUG_DISTURBANCE 0
#define DEBUG_REPRO 0
#define DEBUG_READ 0  
#define DEBUG_UPDATE_SITES 0
#define DEBUG_INTEGRATOR 0
#define DEBUG_GROWTH_RATES 0
#define DEBUG_WN_UPTAKE 0
/**************************************/
/*** ACCOUNTING                     ***/
/**************************************/
/* minimum sizes used in calc of patch site chars */
#define MIN_DBH_CLASS 0.0 /* min dbh (cm) class for dbh calc */
#define MIN_HGT_CLASS 0.0 /* min h(m) class used for dbh calc */

/**************************************/
/*** PRINTING                       ***/
/**************************************/
#define PRINT_OUTPUT_FREQ 60 /*timesteps bewteen output, 1=1month */
#define PRINT_RESTART_FREQ 120    /*freq of printing restart files in NSUB units*/
#define LONG_PATCH_FILE 0 
#define LONG_CD_FILE 0 
#define LONG_FP_FILE 0 
#define LONG_LIGHT_FILE 0 
#define LONG_COHORT_FILE 0 

/*********************** Variable Definitions  *******************/

struct hlist{ /* list of tree height data for site */ 
   float hgt;
   struct hlist *next;
};

struct cohort{
   unsigned int pft;         /* pft number                           */
   int pt;                   /* physiology 1=c3 2=c4                 */
   float n;                  /* number of individuals in cohort      */
   float dbh;                /* dbh in cm                            */
   float hite;               /* height in meters                     */
   float b;	     	     /* total biomass per indiv              */
   float babove;	     /* total above ground biomass           */
   float bbelow;	     /* total below ground biomass           */
   float balive;	     /* total living biomass per indiv       */
   float bdead;	             /* dead biomass per indiv               */
   float bincr;	             /* ch in biomass per indiv              */
   float b_old;	             /* ch in biomass per indiv              */
   float bsw;                /* sapwood in stem and roots */
   float bl; 		     /* leaf biomass per indiv               */
   float bls;                /* leaf biomass in storage "virtual leaves" */    
   float bs;	   /* structural biomass per indiv stem + structual roots */
   float bstem;	             /* stem biomass per indiv               */
   float br;	             /* fine root biomass per indiv    */
   float leaf_area;          /* leaf area of plant                   */
   float lai;                /* leaf area index of plant             */
   float lite;               /* light level for cohort               */
   float gpp;		    /* net primary PER PLANT! Kg/plant over timestep */
   float npp;		    /* net primary PER PLANT! Kg/plant over timestep */
   float npp2;	     /* net primary PER PLANT! Kg/plant over timestep */
   float gpp_max;
   float npp_max;
   float resp;	             /* plant tissue respiration      */
   float gr_resp;	     /* plant growth respiration      */
   float md;	             /* plant tissue maintenence kg/plant over timestep */
   float An_max;
   float An_o;
   float An_c;
   float fs_open; /*fraction of month with stomates open (dimensionless)*/   
   float fsn;     /* degree of n limitation (dimensionless)*/
   float fsw;     /* degree of water limitation (dimensionless)*/
   float fsn_a[N_OUT+1];   /* arrays of within timestep fsn */
   float fsw_a[N_OUT+1];   /* arrays of within timestep fsw */
   float fso_a[N_OUT+1];   /* arrays of within timestep fsopen */
   float carbon_balance;     /* relative carbon balance: ratio of bl to bl* */
   float cbr[N_CLIMATE];           /* carbon balance ratio  */
   float cbr_bar[N_CLIMATE];       /* running average carbon balance ratio  */
   float payment_to_Nfixers; /* carbon flux to symbionts*/
   float p[2];	             /* reproduction seed and clonal */
   int status;               /* growth status of plant */

   float previous_size[12]; /*history of total biomass*/

   /*height profile fields*/
   float density;            /* density of indivs in interval */
   float biomass_density;    /* equiv biomass density in interval */
   float ub;                 /* upper bound of cohort height */
   float lb;                 /* lower bound of cohort height */
   float ag_biomass_density; /* agb of cohort */
   float total_density;         /* total denisty of indivs at cohorts height*/
   float plant_ag_biomass_density; /*total agb density at cohorts height*/
   float plant_biomass_density; /* total biomass density at cohorts height */
   float ubt;                 /* upper bound of density bin */
   float lbt;                 /* lower bound of biomass density bin */
   float observed_total_density;/* height profile data (n) in model bins*/
   float sum_trees;           /*number of obs.trees that fall in cohort height interval*/
   int use_for_interval;      /*logical flag used for binning*/

   /*water fields*/
   float water_uptake;
   float water_stress;
   float E_o;
   float E_c;

   /* variables needed for integration */
   float dndt;                /* time derivative of cohort size */
   float dhdt;                /* time derivative of height */
   float ddbhdt;              /* time derivative of dbh */
   float dbalivedt;           /* time derivative of total living biomass */
   float dbdeaddt;            /* time derivative of dead biomass */
   float dbswdt;              /* time derivative of living stem biomass */
   float dbsdt;               /* time derivative of total stem biomass */
   float dbldt;               /* time derivative of leaf biomass */
   float dblsdt;              /* time derivative of virtual leaf biomass*/
   float dbrdt;               /* time derivative of root biomass */

   /*nitrogen fields*/
   float nitrogen_uptake;
   float N_uptake_pot;
   float N_uptake_shut;

/* element numbers in integration array */
   int n_e;      /* n */
   int dbh_e;      /* trunk diameter */
   int balive_e; /* living carbon */
   int bdead_e;  /* dead carbon (heartwood) */
  
   /*linked list fields*/
   struct cohort *taller;      /* pointer to next tallest cohort     */
   struct cohort *shorter;     /* pointer to next shorter cohort     */
   struct cohort *csptaller;   /* ptr to next tallest cohort of conspecifics */
   struct cohort *cspshorter;  /* ptr to next shorter cohort of conspecifics */
   struct patch *patchptr;     /*pointer to patch that cohort is in*/
   struct site *siteptr;       /*pointer to site that cohort is in*/

};

struct patch{
   float age;                /* patch age in years                   */
   float area;               /* patch area                           */
   char old_address[20];
   struct cohort *tallest;   /* pointer to patch's tallest cohort    */
   struct cohort *shortest;  /* pointer to patch's shortest cohort   */
   struct patch *older;      /* pointer to next older patch   */
   struct patch *younger;    /* pointer to next shorter patch   */   
   struct site *siteptr;     /* pointer to the site that the patch is in*/

   float total_plant_nitrogen_uptake; /*total N uptake by plants (kgN/yr)*/
   float water;          /*total plant available water in mm*/
   float w[N_OUT+1];  /* array of water values for the month */
   float theta;           /* relative water availabiility */
   float plant_water_uptake; /*total plant water uptake in mm*/
   float plant_water_demand;          /*total plant water uptake in mm*/
   float perc;     /* rate of percolation */
   float soil_evap;     /* rate of evap */

   float pft_biomass[N_PFT]; /*biomass of each pft in patch (kgC/m2)*/
   float pft_ag_biomass[N_PFT]; /* agb of each pft in patch (kgC/m2)*/
   float plant_ag_biomass;   /*total above gnd biomass in patch (kgC/m2)*/
   float plant_biomass;   /*total plant biomass in patch (kg/m2)*/
   float basal_area; /* total stem basal area in patch (cm2/m2)*/
   float pft_basal_area[N_PFT];  


   float lai;                     /* leaf area index of patch             */
   float gpp;
   float npp;                /* npp */
   float npp2;               /* npp */
   float nep;                /* nep */

   float aa_gpp;                  /* annual average gpp                    */
   float aa_npp;                  /* annual average npp                    */
   float aa_npp2;                 /* annual average npp                    */
   float aa_nep;                  /* annual average nep                    */
   float soil_c;                  /* total carbon in soil pools            */
   float total_c;                 /* total c                               */
   float rh;                      /* heterotrophic respiration (kgC/m2/yr) */
   float aa_rh;                   /* annually averaged  rh     (kgC/m2/yr) */  
   float litter;                  /* litter flux               (kgC/m2/yr) */
   float repro[N_PFT];            /* array containing seedlings born since last cohort created */  

  /*disturbance values*/
   float fuel;   
   float ignition_rate;   
   float lambda1[N_SUB];  /* site avgd fire dist rate during the year */
   float disturbance_rates[N_DIST_TYPES];
   float disturbance_rate;

   unsigned int dist_type;     /* disturbance dist_type id. dist_type = 1 if fire was last disturbance, equals 0 otherwise, i.e. if treefall was last disturbance*/
 
  /* profiles of various quantities */
   float pft_density_profile[N_PFT][N_DBH_BINS];
   float pft_basal_area_profile[N_PFT][N_DBH_BINS];
   float pft_agb_profile[N_PFT][N_DBH_BINS];
   int fuse_flag; 

   float dadt; /* rate of change of patch age = 1! */
   float dpdt; /* rate of change of patch area */
   float dwdt; /* rate of change of available soil water*/
   float dnppcdt;  /* rate of change of culm npp */
   float drhcdt;   /* rate of change of culm rh  */
  

   /*1-Box Lignin, 4-Box C, 2-BOX N Century*/
   /*soil carbon and nitrogen pools*/
   float mineralized_soil_N; /*mineralized soil nitrogen*/
   float min_N[N_OUT+1];  /* array of N values for montth */
   float fast_soil_N; /*Fast soil nitrogen pool (kg N)*/
   float fast_soil_C; /*Fast soil carbon pool (kg C)*/
   float structural_soil_C; /*Structural soil pool (kg C)*/
   float slow_soil_C; /*Slow soil carbon pool (kg C)*/
   float passive_soil_C; /*Passive soil Carbon (kg C)*/
   float structural_soil_L; /*structural soil lignin content (kg)*/
   float A; /* decomposition rate scalar */
   float fstd; /* fractional rate of structural C decomposition */

   /* monthly pool loadings */
   float ssc_in;
   float ssl_in;
   float fsc_in;
   float fsn_in;
 
   /* current element numbers in integration array */
   int wa_e;   /*patch water */
   int fsc_e;  /*patch fast soil carbon */
   int stsc_e; /*patch structural soil carbon*/
   int ssc_e;  /*patch slow soil carbon */
   int psc_e;  /*patch passive soil carbon*/
   int msn_e;  /*patch mineralized soil nitrogen*/
   int fsn_e;  /*patch fast soil N*/
   int ssl_e;  /*patch str soil L*/

   int okint;
};

/* Type : SiteData contains site-related environmental data */
typedef struct{

   char filename_mech[STR_LEN]; /*name of mech.in file*/
   char pathname_mech[STR_LEN]; /*name of path to mech.in files, machine dep*/
   float area;     /*area of site in m2*/
   float pet[12];      /*potential evapotranspiration (mm)*/
   float pet_average;
   float precip[12];   /*monthly rate of rainfall per unit area */
   float precip_average;  /* avg. yrly precip mm/yr */
   float temp[12];    /* monthly temperature deg C. */
   float temp_average; /* average annual temperature deg. C.*/
   float L_top;    /* Light at the top of the canopy*/
   float Rn_top;   /* Net Radiative flux at the top of the canopy*/
   int soil_type;
   float soil_depth; /* depth of soil in mm */
   float soil_temp[12];  /*avg mnthly soil temp (C)*/
   float soil_temp_average; /*annual average soil temp (C)*/
   float N_conc_in_rain; /*concentration of N in rain water*/
   float clearance_fraction; /* fraction of biomass retained following dist */
   float loss_fraction[N_DIST_TYPES]; /*array of loss fractions following disturbance*/
   int vtype;        /* vegetation class (giss matthews) */

   float theta_max;  /* volume of water in saturated soil mm mm^-3 */
   float k_sat;      /* conductivity of saturated soil (yr^-1) */ 
   float tau;        /* conductivity exponent */
   float soil_evap_conductivity;  /* soil conductivity for evap water loss */
   float olson_carbon;    /*potential above ground carbon pool (Kg/m^2)*/
   float miami_npp;          /*potential annual net primary production (Kg/m^2/yr)*/
   float soil_carbon;  /*potential soil carbon pool (Kg/m^2)*/
   float uhl_age[5];
   float uhl_ntrees[5];
   float uhl_nsdlings[5];
   float uhl_mtrees[5];
   float uhl_msdlings[5];     
   float soil_moisture[12]; /* montly soil moisture data in metres */
   float soil_moisture_average; /* average soil moisture data in metres */
   struct hlist *first; /* pointer to tallest tree at the site  */    

   /* potential photosynthesis and evap*/
   float light_levels[N_LIGHT][PT];
   float Ao[N_LIGHT][N_CLIMATE][PT]; /* pot. net photosynthesis (gC/m2/mo)*/
   float Eo[N_LIGHT][N_CLIMATE][PT];  /* potential transpitation (gW/m2/mo)*/
   float Ac[N_LIGHT][N_CLIMATE][PT]; /* pot. psyn when shut (g/m2/mo)*/  
   float Ec[N_LIGHT][N_CLIMATE][PT];  /* pot. transp when shut (gW/m2/mo)*/
   float tf[N_CLIMATE][PT];  /* value of temp function (used in resp calc)*/

   /* N.A. stuff*/
   float first_frost;
   float last_frost;
   float growing_season_length;

} *SiteData;

struct site{
   char name[STR_LEN];
   int   soi;                     /* site of interest flag 1=Y 0=N */
   float lat;
   float lon;
   SiteData sdata;                /* structure containing site-related data */
   struct patch *oldest_patch;    /* pointer to oldest patch at the site  */
   struct patch *youngest_patch;  /* pointer to yngest patch at the site  */
   struct patch *newpatch;        /* pointer to patch to be spawned */ 
   struct site *next_site;        /* pointer to next site */

   int integration_flag;  /*successful integration = 0,  or not*/
   short use_for_fitting;         /*1=use site for fitting, 0=do not ""*/     

  float plant_biomass;        /* plant biomass               (kgC/m2) */
  float plant_ag_biomass;     /* plant above-gnd biomass     (kgC/m2) */
  float basal_area;           /* stem basal area            (cm2/m2)  */  
  float pft_biomass[N_PFT];   /* biomass per plant type  (kgC/m2) */
  float pft_ag_biomass[N_PFT];/* above-ground biomass per plant type (kgC/m2)*/
  float pft_basal_area[N_PFT];/* stem basal area per plant type (cm2/m2)*/
  float total_c;              /* total c                             */
  float soil_c;               /* soil carbon          (kgC/m2) */  
  float lai;                  /* leaf area index */
  
  /*carbon fluxes*/
  float gpp;                  /* gpp (kgC/m2/yr) */
  float npp;                  /* npp (kgC/m2/yr)*/
  float npp2;                 /* npp (kgC/m2/yr)*/
  float nep;                  /* nep (kgC/m2/yr) */
  float  rh;                   /* heterotrophic respiraction */
  /* annually averaged carbon fluxes  */
  float aa_gpp;
  float aa_npp;                  
  float aa_npp2;                  
  float aa_nep;                   
  float aa_rh; 
  float litter;
  
   float water;                   /* site level water fields */
   float theta;                   /* site level water fields */
   float plant_water_uptake;      
   float plant_water_demand;
   float perc;
   float soil_evap; 
  
   
   float disturbance_rates[N_DIST_TYPES];
   float treefall_disturbance_rate;  /* site avg treefall dist rate */
   float fire_disturbance_rate;      /* site avg fire dist rate */
   float lambda1[N_SUB];  /* site avgd fire dist rate during the year */
   unsigned int dist_type;      /* disturbance dist_type id.*/
   float disturbance_rate; /*site total dist rate*/
   int fire_flag;                    /* fire occurenace flag */
   float fuel;                       /* site level fuel */
   float ignition_rate;              /* site level ignition rate */

   /* profiles of various quantities */
   float pft_density_profile[N_PFT][N_DBH_BINS]; 
   float pft_basal_area_profile[N_PFT][N_DBH_BINS]; 
   float pft_agb_profile[N_PFT][N_DBH_BINS];
   float density_profile[N_DBH_BINS]; 
   float basal_area_profile[N_DBH_BINS]; 
   float agb_profile[N_DBH_BINS]; 
   
   unsigned int create_patch;     /* flag indicating creation of newpatch */
   float pft_biomass_compare[N_PFT]; /*biomass of each pft at site (kgC/m2) updated only every compare freq for use to check if site is finished equilibrating*/
   
   unsigned int finished;       /*if=1, then site is equilibrated*/
   float time_finished;          /*the year when the site finished being integrated*/
  /*1-Box Lignin + 4-Box C + 2-Box N CENTURY*/
  /*soil pools*/
  float mineralized_soil_N;/*total mineralized soil nitrogen (kgN)*/
  float fast_soil_C;         /*total Fast Soil Carbon (kgC)*/
  float fast_soil_N;         /*total fast soil N (KgN)*/
  float structural_soil_C;   /*total Structural Soil Carbon (kgC)*/
  float slow_soil_C;         /*total Slow Soil Carbon (kgC)*/
  float passive_soil_C;      /*total Passive Soil Carbon (kgC)*/
  float structural_soil_L;   /*total Str Soil Lignin (kg)*/

};

struct soi{ /* site of interest */
   char name[STR_LEN];
   float lat;
   float lon;
   struct soi *next_soi;        /* pointer to next soi */
};

/* Type : UserData contains constants and data vectors */
typedef struct{
  char base_filename[STR_LEN]; /*input and output file base name*/
  int number_of_sites;     /*number of sites modeled*/
  /* integration structures paramters and coefficients */ 
  float deltat;
  struct patch *current_patch;  /* current patch being integrated */
  struct site *current_site;  /* current site of patch being integrated */
  int ok;    /* metplex flag */
  float tt;  /* metplex flag */
  unsigned int time_period; /*within year timestep being integrated*/

 /* model parameters */
  float gpp_max;      /* max gpp per unit carbon kg kg^-1 m^-2 yr^-1*/ 
  float k_lite;       /* light saturation coefficient */
  float L_extinct;    /* light extinction per unit leaf bsiomass*/
  float Rn_extinct;   /* Net Radiative flux extinction per unit leaf biomass*/
  float m2;  /* rate at which mortality declines with bl */
  float m3;  /* mortality at zero bl */
  float m1; /* slope of wood density related mortality */
  float sd_mort;       /* seedling mortality */           
  float wup_max;  /* max water uptake per unit root carbon kg kg^-1 yr^-1 */
  float water1;   /* water uptake/stress parameter */
  float nitrogen1;   /* plant nitrogen uptake/stress parameter */
  float nitrogen2;  /* microbial nitrogen uptake/stress parameter */  
  int patch_freq;       /* freq of spawning patches */
  float seed_rain[N_PFT];  /* open recruitment */
  float sapwood_fraction;  /* fraction of stem that is living (sapwood) */ 
  float theta_microbes;    /*exponent used to shut down nitrogen uptake by microbes*/
  float theta_plants;      /*exponent used to shut down water and nitrogen uptake by plants*/
 
  float An_max[N_PFT];               /*(g/m2/mo)*/
  float Vm_max[N_PFT];
  float leaf_life_span[N_PFT];       /*(months)*/
  float specific_leaf_area[N_PFT];    /*(m2/g)*/
  float Fm_leaf[N_PFT];               /* leaf litter metabolic fraction */
  float Fm_sd[N_PFT];               /* leaf litter metabolic fraction */
  float Fm_stem;                         /* stem litter metabolic fraction */
  float Fm_root;                         /* root litter metabolic fraction */

  /*disturbance params*/
  float treefall_hite_threshold;   /* disturbance threshold height (m) */ 
  float treefall_s_ltht[N_PFT];  /* survivorship from disturbance below hite threshold */
  float treefall_s_gtht[N_PFT];  /* survivorship from disturbance above hite threshold */
  float treefall_disturbance_rate;
  float max_patch_age;      /* maximum patch age */
  float fire_hite_threshold;
  float fire_max_disturbance_rate;
  float fp1;     /* fuel parameter used in fire dist rate calculation */
  float k_fw;    /* water paramter used in fire dist rate calculation */

  /* Nitrogen Model coefficients */
  float smoke_fraction;    /*fraction of biomass lost as smoke*/
  float NC_perc_coeff;     /*coefficient for hydrologic loss of N and C*/
  float fraction_of_GPP_to_Nfixers; /* payment for N fixers*/
  int Nfixer[N_PFT];  /*1=nfixer, =non_nfixer*/

  int phenology[N_PFT]; /*phenology flag: 0=evergreen, 1=drought-deciduous, 2=cold-deciduous*/
  int pt[N_PFT];  /* photsynthetic physiology of pft 1=c3 2=c4 */
  float m[N_PFT];   /*fraction dispersal is global*/
  float q[N_PFT];  /* ratio of root to leaf biomass */ 
  float qsw[N_PFT];  /* ratio of root to leaf biomass */ 
  float agf_bs; /* fraction of total structural that is above ground */ 
  float rho[N_PFT];  /* wood density values g cm^-3  */ 
  float rho_max;        /* maximum wood density */ 
  float alpha[N_PFT][NCMPT]; /* decay rates of plant carbon pools */
  float beta[N_PFT][NCMPT];  /* resp rates of plant carbon pools */
 
  float max_dbh[N_PFT];  /* size at which height growth stops */
  float r_fract[N_PFT];  /* fraction of excess c going to seed repro */
  float c_fract[N_PFT];  /* fraction of excess c going to clonal repro */
  float hgt_min[N_PFT];  
  float bl_min[N_PFT];  
  float bs_min[N_PFT];  
 
  /*CENTRUY 4BOX*/
  float l2n_stem;
  float l2n_root;
  float l2n_leaf;
  float l2n_sd;
  float c2n_leaf[N_PFT]; /*(g/g)*/
  float c2n_recruit[N_PFT];
  float c2n_sd[N_PFT]; /*(g/g)*/
  float c2n_stem;
  float c2n_root;
  float c2n_structural;
  float c2n_slow;
  float c2n_passive;

  int mask[180][360];             /* land sea mask*/
  float grid_cell_area[180][360]; /* grid-cell areas */ 
  int num_sois;                   /* number of sites-of-interest */ 

  /* pointers */
  struct soi *first_soi; /* pointer to linked list of sites-of-interest */
	
} *UserData;


/********************** Function Prototypes *************************/
/*control functions*/
void model(struct site **first_site, UserData data, int sflag);
int model_site (int lat,int lon, int counter,UserData data);
void ecosystem_dynamics(unsigned int t, float t1, float t2, struct site **first_site, UserData data);
void integrate_state_variables(struct site **siteptr, unsigned int t, float t1, float t2, UserData data);
int integrate_patch(struct patch **patchptr,unsigned int time_step, float t1, float t2, UserData data);

/*init functions*/
void init_data(UserData data);
void init_patches(struct site **siteptr, UserData data);
void init_cohorts(struct patch **patchptr,UserData data);
void init_sites(struct site **firsts, UserData data);
void init_soi(struct soi **firstsoi, UserData data); /* define sites of interest*/
void reinit_sites(struct site **firsts, UserData data);

/*basic cohort and patch functions*/
void create_patch(struct site **siteptr, struct patch **pnewp, int dist_type, float age, float area, float water, float fsc, float stsc, float stsl, float ssc, float psc, float msn, float fsn, UserData data);
void create_cohort(unsigned int pft, float n, float hite, float dbh, float balive, float bdead, struct patch **patchptr, UserData data);
struct patch *next_older(struct patch *current, float *stp);
struct cohort *next_taller(struct cohort *current, float *stp);
void sort_cohorts(struct patch **patchptr, UserData data);
void terminate_cohorts(struct patch **patchptr, UserData data);
void recruitment(unsigned int t, struct patch **patchptr, UserData data);
void fuse_cohorts(struct patch **patchptr, UserData data);
void insert_cohort(struct cohort **pcurrentc, struct cohort **ptallest, struct cohort **pshortest, UserData data);
void spawn_patches(unsigned int t, struct site **siteptr, UserData data);
void terminate_patches(struct site **siteptr, UserData data);
void fuse_patches(unsigned int t, struct site **siteptr, UserData data);
void fuse_2_patches(struct patch **patchptr1, struct patch **patchptr2, UserData data);
void patch_pft_size_profile(struct patch **pcp,int nbins, UserData data);
void print_patch_linked_list(struct site **siteptr, UserData data);
void print_cohort_linked_list(struct patch **patchptr, UserData data);
int is_soi(float lat, float lon, UserData data); /* flag sites of interest */

void copy_cohort(struct cohort **currentc, struct cohort **copyc);
void split_cohorts(struct patch **patchptr, UserData data);

void calculate_patch_disturbance_rates(unsigned int t, struct patch **current_patch, UserData data);


/*misc functions*/
void light_levels(struct patch **patchptr, UserData data);
void modeled_height_distribution(struct site **fsite, UserData data); 
float height_compare(double *ssq, int *n, struct site **fsite, UserData data);float ntrees_over_2m(struct patch **patchptr); 
float mtrees_over_2m(struct patch **patchptr, UserData data); 
void pft_patch_size_profile(struct patch **pcurrentp, int nbins, UserData data);
void observed_height_distribution(struct site **fsite, UserData data);
void pft_site_size_profile(struct site **pcs, int nbins, UserData data);
void update_biomass(struct site **siteptr, UserData data);
void update_npp(struct site **siteptr, UserData data);
void update_soil_pools(struct site **siteptr, UserData data);
/*for use in auto stopping*/
unsigned int check_if_finished(struct site **firsts, unsigned int timet, UserData data);
void compare_pft_biomass(struct site **siteptr, struct site **firsts, UserData data);
void update_site_biomass_compare(struct site **siteptr);
void update_sites(unsigned int t, struct site **firsts,  UserData data);
float bnldev(float pp, int n, long *idum);
void update_water(struct site **siteptr, UserData data);

/*bio/biogeochem functions*/
void npp(struct cohort **cohortptr, UserData data);
void reproduction(unsigned int t, struct site **siteptr, UserData data);
void update_previous_size(int t, struct site **siteptr, UserData data);
void phenology(int t, struct site **siteptr, UserData data);
void Growth(struct patch **pcurrentp, struct cohort **pcurrentc, float time, UserData data);
void Growth_Derivatives(struct patch **pcurrentp, struct cohort **pcurrentc, float time, UserData data);
void Growth_error(struct patch **pcurrentp, struct cohort **pcurrentc, float time, UserData data);
void Litter(struct patch **pcurrentp, float time, UserData data);
void plant_water_and_nitrogen_relations(struct cohort **cohortptr, UserData data);
float nitrogen_demand_function(struct cohort **pcurrentc, struct patch **pcurrentp, float time, UserData data);

/*allometry functions*/
float Dbh(struct cohort **pcurrentp, UserData data);
float Hite(struct cohort **pcurrentp, UserData data);
float Bdead(struct cohort **pcurrentp, UserData data);
float Bleaf(struct cohort **pcurrentp, UserData data);
float Bd2Dbh(struct cohort **pcurrentc, UserData data);
float dHdBd(struct cohort **pcurrentp, UserData data);
float dDbhdBd(struct cohort **pcurrentp, UserData data);
float dDbhdBl(struct cohort **pcurrentc, UserData data);

/* mortality and disturbance funcitons*/
float survivorship_from_disturbance(int q,struct cohort **pcurrentc, UserData data);
void mortality(struct site **siteptr, UserData data);
float mortality_rates(struct cohort **pcurrentc, UserData data);
void calculate_site_disturbance_rates(unsigned int t, struct site **currents, UserData data);
void calculate_patch_distubance_rates(unsigned int t, struct site **currents, UserData data);
void fire_frequency(unsigned int t, struct site **siteptr, UserData data);

/* reading routines*/
void read_input_data_layers(UserData data);
void read_sois(UserData data);
void read_mask(UserData data);
void read_grid_cell_area(UserData data);
void read_soil_chars(struct site **first_site);
void read_mechanism_lut(struct site **siteptr, UserData data);
void read_precip(struct site **first_site);
void read_soil_temp(struct site **first_site);
void read_temp(struct site **first_site);
void read_patch_distribution(struct site **siteptr, UserData data);
void read_cohort_distribution(struct site **siteptr, struct patch **patchptr, char *paddress1, UserData data);


/*update and print routines*/
void calculate_site_water(struct site **siteptr, UserData data);
void calculate_site_biomass(struct site **siteptr,UserData data);  
void calculate_site_cfluxes(struct site **siteptr, UserData data);
void calculate_site_soil_pools(struct site **siteptr, UserData data);
  /*calculate total_sc*/
void calculate_patch_cfluxes(struct patch **patchptr,UserData data);
void calculate_patch_biomass(struct patch **patchptr,UserData data);

void print_diagnostics(unsigned int time, struct site **siteptr,UserData data);
void print_system_state(unsigned int time,struct site **siteptr,UserData data);
void print_site_chars(struct site **fsite, UserData data);
void print_site_biomass(unsigned int time, struct site **siteptr, UserData data);
void print_site_cfluxes(unsigned int time, struct site **siteptr, UserData data);
void print_site_water(unsigned int time, struct site **siteptr, UserData data); 
void print_site_soil_pools(unsigned int time, struct site **siteptr, UserData data);
void print_nitrogen_budget(unsigned int time, struct site **currents, UserData data);
void print_patches(unsigned int time, struct site **siteptr, UserData data);
void print_patch_cfluxes(unsigned int time, struct site **siteptr, UserData data);
void print_patch_light_levels(unsigned int time,struct site **siteptr,UserData data);
void print_patch_size_profile(unsigned int t, struct patch **pcurrentp, int nbins, UserData data);
void print_site_size_profile(unsigned int t, struct site **pcurrents, int nbins, UserData data);
void print_patch_water(unsigned int time, struct site **siteptr, UserData data);
void print_cohorts(unsigned int time, struct site **siteptr, UserData data);

/* print region routines */
void print_region_temp(struct site **siteptr,  UserData data);
void print_region_soil_temp(struct site **siteptr,  UserData data);
void print_region_diagnostics(unsigned int t,struct site **siteptr,  UserData data);
void print_region_precip(struct site **siteptr,  UserData data);
void print_region_mech_E(struct site **siteptr,  UserData data);
void print_region_mech_An(struct site **siteptr, UserData data);
void print_region_mask(struct site **siteptr, UserData data);
void print_region_soil_chars(struct site **siteptr,  UserData data);
void print_region_agb(unsigned int t, struct site **siteptr,  UserData data);
void print_region_biomass(unsigned int t, struct site **siteptr,  UserData data);
void print_region_pft_biomass(unsigned int t, struct site **siteptr,  UserData data);
void print_region_max_height(unsigned int t, struct site **siteptr, UserData data);
void print_region_integration_flag(unsigned int t, struct site **siteptr, UserData data);
void print_region_carbon_fluxes(unsigned int t, struct site **siteptr, UserData data);
void print_region_water(unsigned int t, struct site **siteptr, UserData data);

void print_region_basal_area(unsigned int t, struct site **siteptr, UserData data);
void print_region_soil_C_and_N(unsigned int t, struct site **siteptr, UserData data);
void print_region_disturbance_rates(unsigned int t, struct site **siteptr, UserData data);
void print_region_water_losses(unsigned int t, struct site **siteptr, UserData data);
void print_region_lai(unsigned int t, struct site **siteptr, UserData data);


/************************************************************************/
/*********************** END OF FILE                          ***********/
/************************************************************************/

