/*  odeint.c
 *   Copyright (C) 2001 Paul Moorcroft, G.C. Hurtt and S.W. Pacala
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*5/1/00 modified to include euler step if stiff integration fails*/
#include "ed.h"
#include "vector.h"
#include "llnltyps.h"  /* definitions of real, integer, bool, TRUE, FALSE */
#include "cvode.h"     /* main CVODE header file                          */
#include "iterativ.h"  /* contains the enum for types of preconditioning  */
#include "cvspgmr.h"   /* use CVSPGMR linear solver each internal step    */
#include "dense.h"     /* use generic DENSE solver for preconditioning    */
#include "llnlmath.h"  /* contains SQR macro                              */

#define RTOL    1.0e-3         
#define FLOOR   0.000001
#define ATOL    RTOL*FLOOR   /* scalar absolute tolerance */
#define MAX_ODE_STEPS 1000   /* max # of substeps  by stiff ode integrator */ 
                             /* default = 500 */ 
#define NCSV 4               /* Number of cohort state variables */
#define NPSV 8               /* Number of patch state variables */     
#define ZERO RCONST(0.0)     /* constant definition needed for norm calc */


int integrate_patch(struct patch **patchptr,unsigned int time_step, float t1, float t2, UserData data);
static void PrintFinalStats(long int iopt[],struct patch **patchptr);

/* Functions Called by the CVODE Solver */
static int Precond(integer N, real tn, N_Vector u, N_Vector fu, bool jok,
                   bool *jcurPtr, real gamma, N_Vector ewt, real h,
                   real uround, long int *nfePtr, void *P_data,
                   N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3);
static int PSolve(integer N, real tn, N_Vector u, N_Vector fu, N_Vector vtemp,
                  real gamma, N_Vector ewt, real delta, long int *nfePtr,
                  N_Vector r, int lr, void *P_data, N_Vector y);
static void f(integer N, real t, N_Vector u, N_Vector udot, void *f_data);

void Water_and_Nitrogen_Uptake(struct site **siteptr, struct patch **patchptr, unsigned int time_period, float time, N_Vector u, N_Vector udot,UserData data);
void Dwdt(struct site **siteptr, struct patch **patchptr, float time, N_Vector u, N_Vector udot, UserData data);
void Dsdt(struct site **siteptr, struct patch **patchptr, unsigned int time_period, float time, N_Vector u, N_Vector udot,UserData data);

void Dwdt_error(struct site **siteptr, struct patch **patchptr, float time, N_Vector u, N_Vector udot, UserData data);
void Dsdt_error(struct site **siteptr, struct patch **patchptr, unsigned int time_period, float time, N_Vector u, N_Vector udot,UserData data);

/* Century: function prototypes of functions called in Dsdt*/
float A_function(struct site **siteptr,struct patch **patchptr, unsigned int time_period,UserData data);
/****************************************************************************/

void integrate_state_variables(struct site **siteptr, unsigned int t, float t1, float t2, UserData data){ /*loop over patches and integate state vars in each */
  struct site *cs;
  struct patch *cp;   
  FILE *outfile;
  char filename[STR_LEN];

  strcpy(filename,data->base_filename);
  strcat(filename,".ed");

  if(!(outfile=fopen(filename,"w"))){
      fprintf(stdout,"ed: Can't open file: %s \n",filename);
      while(getchar() != '\n');
  }

  cs= *siteptr;
  cp = cs->youngest_patch;
  while(cp!=NULL){ 
    cp->okint= integrate_patch(&cp,t,t1,t2,data);  
    if(cp->okint != 0) 
      fprintf(outfile,"isv: exited from sode patch= %p ok=%d\n",cp,cp->okint);
    /* age patches */      
    cp -> age += data->deltat; 
    cp=cp->older;
  }
  fclose(outfile);
}
/****************************************************************************/
int integrate_patch(struct patch **patchptr, unsigned int time_step, float t1, float t2, UserData data){
 
  real abstol, reltol, t, tout, ropt[OPT_SIZE];  
  long int iopt[OPT_SIZE];
  void *cvode_mem;
  int iout, flag;
  N_Vector u;
  N_Vector dudt;
  N_Vector dudtsave;
  real *udata;    /* pointers used to recover solutions */
  real *dudtdata; /* pointers used to recover solutions */   
  real *dudtsavedata; /* pointers used to recover solutions */   
  int i,cindex;
  int ccnt;  /* number of cohorts in current patch */
  int neq; /* number of variables to be integrated */
  struct patch  *cp;
  struct cohort  *cc;
  FILE *integration_log_file;
  char integration_log_filename[256];
  
  for(i=0;i<OPT_SIZE;i++){ iopt[i]=0; ropt[i]=0.0;}
  iopt[1]=MAX_ODE_STEPS;

  strcpy(integration_log_filename,data->base_filename);
  strcat(integration_log_filename,".integration_log");
  integration_log_file=fopen(integration_log_filename,"a");

  cp = *patchptr;

  /*printf("sode: integrating site %p patch %p age %f \n",cp->siteptr,cp,cp->age); */
  /*printf("cm_sodeint t1= %f t2= %f\n", t1,t2);*/

  data->current_patch= cp; /* assign current patch to patch strucutre in UserData structure data (address of patch to be integrated) */ 
  
  ccnt=0;  
  cc = cp->shortest;
  while(cc != NULL){
    ccnt++;
    /* printf("value of ccnt= %d value of tc= %d \n",ccnt,tc); */
    cc=cc->taller;
  } 
  neq = NCSV*ccnt  + NPSV;

  /* Allocate vector memory, set problem data, initial values, tolerances */
  /* printf("sode: allocating memory ..\n"); */ 
  u = N_VNew(neq, NULL);              /* Allocate u vector  */
  dudt = N_VNew(neq, NULL);           /* Allocate dudt vector  */
  dudtsave = N_VNew(neq, NULL);           /* Allocate dudt vector  */
  abstol= ATOL; reltol=RTOL; 

  /* assign pointers */
  udata = N_VDATA(u);   /*printf("sode: assigning pointers ..\n"); */
  dudtdata = N_VDATA(dudt); /*   printf("sode: assigning pointers ..\n"); */
  dudtsavedata = N_VDATA(dudtsave);/*printf("sode: assigning pointers ..\n");*/
 
  /* load patch elements into integration vector */
  /*printf("sode: loading elements ..\n");*/ 
  udata[cp->wa_e] = cp->water; 
  udata[cp->fsc_e] = cp->fast_soil_C; 
  udata[cp->stsc_e] = cp->structural_soil_C;      
  udata[cp->ssc_e] = cp->slow_soil_C; 
  udata[cp->msn_e] = cp->mineralized_soil_N;
  udata[cp->fsn_e] = cp->fast_soil_N;
  udata[cp->psc_e] = cp->passive_soil_C;
  udata[cp->ssl_e] = cp->structural_soil_L;

#if DEBUG_INTEGRATOR
  printf("t= %f sode: pre age  %f \n",t1,cp->age);
  printf("t= %f sode: pre area  %f \n",t1,cp->area);
  printf("t= %f sode: pre water  %f \n",t1,cp->water);
  printf("t= %f sode: pre fsc  %f \n",t1,cp->fast_soil_C);
  printf("t= %f sode: pre stsc %f \n",t1,cp->structural_soil_C);
  printf("t= %f sode: pre ssc  %f \n",t1,cp->slow_soil_C);
  printf("t= %f sode: pre psc  %f \n",t1,cp->passive_soil_C);
  printf("t= %f sode: pre fsn  %f \n",t1,cp->fast_soil_N);
  printf("fsc_in= %f ssc_in= %f ssl_in %f fsn_in %f \n",cp->fsc_in,cp->ssc_in,cp->ssl_in,cp->fsn_in);
  printf("\n");
#endif
 /* load cohort elements into integration vector */
  /* printf("sode: loading elements ..\n"); */
  cindex=0;             /* cohort index */
  cc = cp->shortest;
  while(cc != NULL){        
#if DEBUG_INTEGRATOR
  printf("t= %f sode: pre cc %p dbh= %f pft= %d n= %f ba= %f bd= %f \n",t1,cc,cc->dbh,cc->pft,cc->n,cc->balive,cc->bdead);
#endif
      cc->n_e = NPSV + cindex;            /* assign cohort elements */
      cc->dbh_e = NPSV + cindex + 1*ccnt;
      cc->balive_e = NPSV + cindex + 2*ccnt;
      cc->bdead_e = NPSV + cindex + 3*ccnt;
      udata[cc->n_e] = cc->n;
      udata[cc->dbh_e] = cc->dbh;
      udata[cc->balive_e] = cc->balive;
      udata[cc->bdead_e] = cc->bdead;
      cindex++;     
      cc=cc->taller;
  }

   /* pass to stiff ode integrator */
   cvode_mem = CVodeMalloc(neq, f, t1, u, BDF, NEWTON, SS, &reltol,
                          &abstol, data, NULL, TRUE, iopt, ropt, NULL);
        
  if (cvode_mem == NULL) { printf("CVodeMalloc failed."); return(1); }
  /* Call CVSpgmr to specify the CVODE linear solver CVSPGMR with
     left preconditioning, modified Gram-Schmidt orthogonalization,
     default values for the maximum Krylov dimension maxl and the tolerance
     parameter delt, preconditioner setup and solve routines Precond and
     PSolve, and the pointer to the user-defined block data.               */
      
  CVSpgmr(cvode_mem, NONE, MODIFIED_GS, 0, 0.0, Precond, PSolve, data);

  f(neq,t1,u,dudt,data);     /* calc and print initial dudt values */    
 
  /* store initial patch element derivatives */

  dudtsavedata[cp->wa_e]= dudtdata[cp->wa_e];
  dudtsavedata[cp->fsc_e]= dudtdata[cp->fsc_e];
  dudtsavedata[cp->stsc_e]= dudtdata[cp->stsc_e];
  dudtsavedata[cp->ssc_e]= dudtdata[cp->ssc_e];
  dudtsavedata[cp->msn_e]= dudtdata[cp->msn_e];
  dudtsavedata[cp->fsn_e]= dudtdata[cp->fsn_e];
  
  dudtsavedata[cp->ssl_e]= dudtdata[cp->ssl_e];

  /* cohort element derivatives */
    cc = cp->shortest;
    while(cc != NULL){        
    dudtsavedata[cc->n_e]= dudtdata[cc->n_e]; 
    dudtsavedata[cc->dbh_e]=dudtdata[cc->dbh_e];
    dudtsavedata[cc->balive_e]=dudtdata[cc->balive_e];
    dudtsavedata[cc->bdead_e]=dudtdata[cc->bdead_e];
    cc=cc->taller;
  }


  /*     printf(" inital f evaluation complete \n\n"); */
  /* loop over output points, call CVode, print results, test for error */
  for (iout=1, tout = t1+data->deltat; iout <= 1; iout++, tout += data->deltat) {
    
    flag = CVode(cvode_mem, tout, u, &t, NORMAL);
        
    if (flag != SUCCESS){ 
      printf("CVode failed, flag=%d site %s %p   patch %p age %6.3f \n", flag,data->current_site->name,data->current_site,data->current_patch,data->current_patch->age); 

      cp->siteptr->integration_flag=flag;
      fprintf(integration_log_file,"INTEGRATION FAILURE: time= %f site= %s flag= %d\n",t1,cp->siteptr->name,cp->siteptr->integration_flag);

  /* print patch element derivatives */
  printf("odeint: t= %f dudt w    %f \n",t1,dudtsavedata[cp->wa_e]);
  printf("odeint: t= %f dudt fsc  %f \n",t1,dudtsavedata[cp->fsc_e]);
  printf("odeint: t= %f dudt stsc %f \n",t1,dudtsavedata[cp->stsc_e]);
  printf("odeint: t= %f dudt ssc  %f \n",t1,dudtsavedata[cp->ssc_e]);
  printf("odeint: t= %f dudt msn  %f \n",t1,dudtsavedata[cp->msn_e]);
  printf("odeint: t= %f dudt fsn  %f \n",t1,dudtsavedata[cp->fsn_e]);
  printf("odeint: t= %f dudt ssl  %f \n",t1,dudtsavedata[cp->ssl_e]);


  /* print cohort element derivatives */
  cc = cp->shortest;
  while(cc != NULL){        
      printf("odeint: t= %f p %p c %p dudt %f \n",t1,cp,cc,dudtsavedata[cc->n_e]); 
      printf("odeint: t= %f p %p c %p dudt %f \n",t1,cp,cc,dudtsavedata[cc->dbh_e]);
      printf("odeint: t= %f p %p c %p dudt %f \n",t1,cp,cc,dudtsavedata[cc->balive_e]);
      printf("odeint: t= %f p %p c %p dudt %f \n",t1,cp,cc,dudtsavedata[cc->bdead_e]);
       printf("\n");
      cc=cc->taller;
  }

      break;
    }
  } /* end loop over iout */
  
  /* load returned elements back into coresponding structures */
  if (flag == SUCCESS){ 
    cp->water = udata[cp->wa_e];
    cp->fast_soil_C = udata[cp->fsc_e];
    cp->structural_soil_C = udata[cp->stsc_e];
    cp->slow_soil_C = udata[cp->ssc_e];
    cp->passive_soil_C = udata[cp->psc_e];
    cp->mineralized_soil_N = udata[cp->msn_e];
    cp->fast_soil_N = udata[cp->fsn_e];
    cp->structural_soil_L = udata[cp->ssl_e]; 

#if DEBUG_INTEGRATOR
  printf("t= %f sode: post age  %f \n",t1,cp->age);
  printf("t= %f sode: post area  %f \n",t1,cp->area);
  printf("t= %f sode: post water  %f \n",t2,cp->water);
  printf("t= %f sode: post fsc  %f \n",t2,cp->fast_soil_C);
  printf("t= %f sode: post stsc %f \n",t2,cp->structural_soil_C);
  printf("t= %f sode: post ssc  %f \n",t2,cp->slow_soil_C);
  printf("t= %f sode: post psc  %f \n",t2,cp->passive_soil_C);
  printf("t= %f sode: post fsn  %f \n",t2,cp->fast_soil_N);
  printf("\n");
#endif

  /* load returned cohort  elements back into coresponding structures */
  /* printf("sode: unloading cohort elements ..\n"); */
    cc = cp->shortest;
    while(cc != NULL){
      cc->n = udata[cc->n_e];
      cc->dbh = udata[cc->dbh_e];
      cc->balive = udata[cc->balive_e]; 
      cc->bdead = udata[cc->bdead_e]; 
      cc=cc->taller;
    }
  } /* endif on sucesss flag */
  else{ /* euler step */
    cp->water *= exp(dudtsavedata[cp->wa_e]*data->deltat);
    cp->fast_soil_C *= exp(dudtsavedata[cp->fsc_e]*data->deltat);
    cp->structural_soil_C *= exp(dudtsavedata[cp->stsc_e]*data->deltat);
    cp->slow_soil_C *= exp(dudtsavedata[cp->ssc_e]*data->deltat);
    cp->mineralized_soil_N *= exp(dudtsavedata[cp->msn_e]*data->deltat);
    cp->fast_soil_N *= exp(dudtsavedata[cp->fsn_e]*data->deltat);
    cp->structural_soil_L *= exp(dudtsavedata[cp->ssl_e]*data->deltat); 

    cc = cp->shortest;
    while(cc != NULL){
      cc->n *= exp(dudtsavedata[cc->n_e]*data->deltat);
      cc->dbh *= exp(dudtsavedata[cc->dbh_e]*data->deltat);
      cc->balive *= exp(dudtsavedata[cc->balive_e]*data->deltat); 
      cc->bdead *= exp(dudtsavedata[cc->bdead_e]*data->deltat); 
      cc=cc->taller;
    }
   f(neq,t1,u,dudt,data);     /* recompute dudt values */    

  } /* end else */
  /* Free memory and print final statistics */  
    N_VFree(u);
    N_VFree(dudt);
    N_VFree(dudtsave);
    CVodeFree(cvode_mem);
    /*PrintFinalStats(iopt,&cp); */ 

    fclose(integration_log_file);

  return flag;   /* return to community dynamics */
}
/****************************************************************************/
/***************** Functions Called by the CVODE Solver *********************/
/* f routine. Compute f(t,u). */
static void f(integer N, real t, N_Vector u, N_Vector udot, void *f_data){

  struct cohort *cc;
  struct patch *cp;
  struct site  *cs;
  UserData data; 
  real *udata, *dudata;         /* pointers to vectors */

  /* Extract constants from data structure */
  data = (UserData)(f_data);  
  
  cs = data->current_site;
  cp = data->current_patch;
  /* printf("f: patch= %p \n",cp); */

  /* assign vector pointers */
  udata = N_VDATA(u);
  dudata = N_VDATA(udot);

  cp->water = udata[cp->wa_e];
  cp->fast_soil_C = udata[cp->fsc_e];
  cp->structural_soil_C = udata[cp->stsc_e];
  cp->slow_soil_C = udata[cp->ssc_e];
  cp->mineralized_soil_N = udata[cp->msn_e];
  cp->fast_soil_N = udata[cp->fsn_e];
  cp->passive_soil_C = udata[cp->psc_e];
  cp->structural_soil_L = udata[cp->ssl_e];
 
  /* unpack cohort elements of u to relevent linked lists */
  /* printf("f: unloading cohort elements ..\n"); */
  cc = cp->shortest;
  while(cc != NULL){
    cc->n = udata[cc->n_e];
    cc->dbh = udata[cc->dbh_e];
    cc->balive = udata[cc->balive_e]; 
    cc->bdead = udata[cc->bdead_e]; 
    cc=cc->taller;
  }

  /* calculate light levels */
  /* printf("f: calulate light levels \n"); */
  sort_cohorts(&cp,data);
  light_levels(&cp,data);

  /* update water and nitrogen uptake rates */
  Water_and_Nitrogen_Uptake(&cs,&cp,data->time_period,t,u,udot,data);

  Dwdt(&cs,&cp,t,u,udot,data); 
 /* update litter loadings */
  Litter(&cp,t,data);
  Dsdt(&cs,&cp,data->time_period,t,u,udot,data);

  /* call growth function */
  /* printf("f: calculating growth function \n"); */
  cc = cp->shortest;
  while(cc!=NULL){
    /* printf("f: cc %p \n",cc); */
          Growth_Derivatives(&cp,&cc,t,data);  
	  cc=cc->taller;
  }
  /* pack cohort linked list results into dudata */
  cc = cp->shortest;
  while(cc != NULL){        
    dudata[cc->n_e] = cc->dndt;
    dudata[cc->dbh_e] = cc->ddbhdt;
    dudata[cc->balive_e] = cc->dbalivedt;
    dudata[cc->bdead_e] = cc->dbdeaddt;
    cc=cc->taller;
  }

  
  return;
}
/*****************************************************************************/
/* Preconditioner setup routine. Generate and preprocess P. */
static int Precond(integer N, real tn, N_Vector u, N_Vector fu, bool jok,
                   bool *jcurPtr, real gamma, N_Vector ewt, real h,
                   real uround, long int *nfePtr, void *P_data,
                   N_Vector vtemp1, N_Vector vtemp2, N_Vector vtemp3)
{ return(0); }

/* Preconditioner solve routine */

static int PSolve(integer N, real tn, N_Vector u, N_Vector fu, N_Vector vtemp,
                  real gamma, N_Vector ewt, real delta, long int *nfePtr,
                  N_Vector r, int lr, void *P_data, N_Vector y)
{ return(0); }
/****************************************************************************/
/*****************************************************************************/
/* Print final statistics contained in iopt */
static void PrintFinalStats(long int iopt[],struct patch **patchptr){
  struct patch *cp;
  cp= *patchptr;
  printf("\nFinal Statistics for integration of patch %p age %f \n\n",cp,cp->age);
  printf("lenrw   = %5ld    leniw = %5ld\n", iopt[LENRW], iopt[LENIW]);
  printf("llrw    = %5ld    lliw  = %5ld\n", iopt[SPGMR_LRW], iopt[SPGMR_LIW]);
  printf("nst     = %5ld    nfe   = %5ld\n", iopt[NST], iopt[NFE]);
  printf("nni     = %5ld    nli   = %5ld\n", iopt[NNI], iopt[SPGMR_NLI]);
  printf("nsetups = %5ld    netf  = %5ld\n", iopt[NSETUPS], iopt[NETF]);
  printf("npe     = %5ld    nps   = %5ld\n", iopt[SPGMR_NPE], iopt[SPGMR_NPS]);
  printf("ncfn    = %5ld    ncfl  = %5ld\n \n", iopt[NCFN], iopt[SPGMR_NCFL]);
}
/*****************************************************************************/
void Water_and_Nitrogen_Uptake(struct site **siteptr, struct patch **patchptr,unsigned int time_period, float time, N_Vector u, N_Vector udot,UserData data){

  struct cohort *cc;
  struct patch *cp;
  struct site *cs;
  real *udata, *dudata;         /* pointers to vectors */
  int pft,pt;
  float water_supply, nitrogen_supply;
  int light_index, time_index;

  cs=*siteptr;
  cp=*patchptr;
  
  /* assign vector pointers */
  udata = N_VDATA(u);
  dudata = N_VDATA(udot);

  /* printf("wnu:\n"); */ 
  /*month index*/
  time_index = data->time_period;


  /*TRANSPIRATION****************************/
  cc=cp->shortest;
  while(cc != NULL){
    pft = cc->pft;
    pt = cc->pt;
    light_index = 0; /*calc light index*/  
    while(cs->sdata->light_levels[light_index][pt] > cc->lite){
      light_index++;
    }
    /*   printf("wnu: transpiration: cc %p li= %d ti= %d \n",cc,light_index,time_index); */

    /*potential transpiration (gH20/m2(leaf)/mon)*/
    cc->E_o = cs->sdata->Eo[light_index][time_index][pt]; 
    /*convert to (kgH20/yr per plant)*/
    cc->E_o *= data->specific_leaf_area[pft]*N_CLIMATE/1000.0;

    /*potential transpiration (gH20/m2(leaf)/mon)*/
    cc->E_c = cs->sdata->Ec[light_index][time_index][pt];
    /*convert to (kgH20/yr per plant)*/
    cc->E_c *= data->specific_leaf_area[pft]*N_CLIMATE/1000.0;

    /*NITROGEN UPTAKE**************************/
    /* set fs_open to 1 and */
    /* calc implied nitrogen uptake (kgN/yr) per plant*/
    cc->fs_open=1.0;
    cc->N_uptake_pot= nitrogen_demand_function(&cc,&cp,time,data);
    /*convert to kgN/m2/yr per plant*/
    cc->N_uptake_pot /= cc->patchptr->area;

    /*nitrogen uptake with stomates shut*/
    cc->N_uptake_shut = 0.0;

    cc=cc->taller;  
  } /* end loop over cohorts */

  
  /* printf("wnu: calculating individual uptake rates \n"); */
    /* calculate actual uptake rates  */  
    cc=cp->shortest;
    while(cc != NULL){

     water_supply=data->water1*data->q[cc->pft]*udata[cp->wa_e]*MASS_OF_WATER;
     nitrogen_supply=data->nitrogen1*cc->br*udata[cp->msn_e];
#if DEBUG_WN_UPTAKE
     printf("wnu: iuptake: p= %p age= %f w= %f n= %f c= %p h= %f pft %d nind= %f br= %f Eo= %f ws= %f ns= %f \n",cp,cp->age,udata[cp->wa_e],udata[cp->msn_e],cc,cc->hite,cc->pft,cc->n/cp->area,cc->br,cc->E_o,water_supply,nitrogen_supply);
#endif
#if WATER_COMPETITION
     cc->fsw = water_supply/(cc->E_o + water_supply);
#else
     cc->fsw = 1.0;
#endif
#if N_COMPETITION
    
     if(cc->N_uptake_pot >0.00)
       cc->fsn = nitrogen_supply/(cc->N_uptake_pot+nitrogen_supply);
     else
       cc->fsn = 1.00;
     /*so that fsn !> 1.0 when plant n demand is < 0*/
#else 
     cc->fsn = 1.0;
#endif
     cc->fs_open = cc->fsw*cc->fsn;

     /*printf("wnu1: fs_open %f\n",cc->fs_open);*/

     cc->water_uptake = cc->bl*(cc->fs_open*cc->E_o + (1.0-cc->fs_open)*cc->E_c); 

    /*recompute n uptake with reduced npp, not simple function like evap
    because allocation in npp dependent*/
     cc->nitrogen_uptake = nitrogen_demand_function(&cc,&cp,time,data);
     cc->nitrogen_uptake/=cc->patchptr->area;      

     /*printf("wnu2: fs_open %f\n",cc->fs_open);*/

     /*printf("%f\n",cc->nitrogen_uptake);*/
  
      cc=cc->taller;
    } /* end loop over cohorts */

    /* printf("wnu: calculating total uptake rates \n"); */
  cp->plant_water_uptake=0.0;
  cp->plant_water_demand=0.0;
  cp->total_plant_nitrogen_uptake=0.0;  
  cc=cp->shortest;
  while(cc != NULL){
#if WATER_COMPETITION
    cp->plant_water_demand += cc->E_o*cc->n;
    cp->plant_water_uptake += cc->water_uptake*cc->n;
#endif
#if N_COMPETITION
    cp->total_plant_nitrogen_uptake += cc->nitrogen_uptake*cc->n;
#endif   
    cc=cc->taller;
  }
#if DEBUG_WN_UPTAKE
  printf("wnu: patch %p age= %f area= %f \n",cp,cp->age,cp->area); 
  cc=cp->shortest;
  while(cc != NULL){
    printf("wnu: cohort %p hite= %f pft= %u fso= %f fsw= %f fsn= %f \n",cc,cc->hite,cc->pft,cc->fs_open,cc->fsw,cc->fsn);
    cc=cc->taller;
  }
  printf("wnu: patch %p total water uptake = %f \n",cp,cp->plant_water_uptake);
  printf("wnu: patch %p total n uptake = %f \n",cp,cp->total_plant_nitrogen_uptake);
#endif 
  return;
  
}
/***************************************************************************/
void Dwdt(struct site **siteptr, struct patch **patchptr, float time, N_Vector u, N_Vector udot, UserData data){ 

/*This function, and those that it calls, must use *w !*/

  /*this function calculates and updates dwdt of a patch*/
  struct patch *cp;
  struct site *cs;
  real *udata, *dudata;         /* pointers to vectors */


  cs=*siteptr;
  cp=*patchptr;
  /* assign vector pointers */
  udata = N_VDATA(u);
  dudata = N_VDATA(udot);

  cp->soil_evap =  0.0; /* not implemented yet */
  
  /*********************************************/
  /*********************************************/
  /* calculate water loss per unit area from patch */
  cp->theta= udata[cp->wa_e]/(cs->sdata->soil_depth*cs->sdata->theta_max);
  if(udata[cp->wa_e] > 0.0) 
    cp->perc=  cs->sdata->k_sat*pow(cp->theta,2.0*cs->sdata->tau+2.0);
  else cp->perc =0.0;
  dudata[cp->wa_e]= (cs->sdata->precip[data->time_period] - 
			   cp->perc - cp->plant_water_uptake/cp->area) - cp->soil_evap;
  
#if DEBUG_INTEGRATOR
  printf("t= %f Dwdt: site = % p patch = %p \n",time,cs,cp);
  printf("t= %f Dwdt: w = %f dwdt = %f \n",time,udata[cp->wa_e],dudata[cp->wa_e]);
  printf("t= %f Dwdt: time period = %d\n",time,data->time_period);
  printf("t= %f Dwdt: precip = %f \n",time,cs->sdata->precip[data->time_period]); 
  printf("t= %f Dwdt: perc = %f \n",time,cp->perc);
  printf("t= %f Dwdt: tot water uptake = %f \n",time,cp->plant_water_uptake/cp->area);
  printf("t= %f Dwdt: evap = %f \n",time,cp->soil_evap);

#endif
}

/**************************************************************************/
void Dsdt(struct site **siteptr, struct patch **patchptr, unsigned int time_period, float time, N_Vector u, N_Vector udot,UserData data){
  real *udata, *dudata;         /* pointers to vectors */

  /***This function must set time derivatives for each soil pool***/  
  /*Simplified 4-Box Century, based on Parton et al 1987 and 1993*/
  /*4 carbon boxes + 2 nitrogen boxes*/
  /*There is no: leaching, other n inputs, or effect of soil type*/
  
  float fast_C_loss=0.00,  fast_N_loss=0.00;
  float structural_C_loss=0.00;
  float slow_C_input=0.00,  slow_C_loss=0.00, slow_N_loss=0.00;
  float mineralized_N_input=0.00, mineralized_N_loss=0.00;
  float N_immobilization_demand=0.00;
  float passive_C_input=0.00, passive_C_loss=0.00, passive_N_loss=0.00;
  float structural_L_loss=0.00;

  float Ls; /*the fraction of structural material that is lignin*/
  float Lc; /*decomp rate reduction due to lignin*/
  float A;  /*decomp rate reduction due to moisture and temperature*/

  struct cohort *cc;
  struct patch *cp;
  struct site *cs;

  /*CENTURY PARAM VALUES*/
  /*values are simple averages from lumped Century Pools*/
  /*Based on Parton et al 1993 GBC*/
  /*1-structural,2-fast,3=slow,4=passive*/
  float K1=4.5,K2=11.0,K3=100.2,K4=0.0;     /*Max decay rate yr^-1; 
					 from Century directly, 
					 or averages of lumped
                                         Century pools*/
 /* K3 is high bc we wanted to added back the n immobilization story
    without dist_typeing the slow pool*/

  /*std values= 1,.3,1,0*/
  float r_fsc=1.0, r_stsc=0.3, r_ssc=1.0, r_psc=0.0; 
  /* respiration rates of soil pools */
					
  cs=*siteptr;
  cp=*patchptr;
 /* assign vector pointers */
  udata = N_VDATA(u);
  dudata = N_VDATA(udot);

  /**************/
  /*OTHER FLUXES*/
  /**************/
  /*calculate commonly used terms once*/
  cp->A=A_function(&cs,&cp, time_period,data);
  A=cp->A;

  /*Slow down of decomp due to fraction of structural material that is lignin*/
  /*From CENTURY Parton et al 1993 GBC 7(4):785-809*/
  Ls=udata[cp->ssl_e]/udata[cp->stsc_e];
  if(udata[cp->stsc_e] > 0.0){ Ls=udata[cp->ssl_e]/udata[cp->stsc_e];
    Lc= exp(-3.0*Ls);
  }
  else{ Ls =0.0; Lc=0.0;}

#if ! N_COMPETITION 
  cp->total_plant_nitrogen_uptake=0.0;
#endif

  mineralized_N_loss = cp->total_plant_nitrogen_uptake;

#if DEBUG_INTEGRATOR
  printf("Dsdt: litter flux in %f plant uptake %f\n",cp->fsn_in,cp->total_plant_nitrogen_uptake);
#endif

#if N_DECOMP_LIMITATION
  /* N immobilized by the decomp of structural material*/
  N_immobilization_demand= A*Lc*K1*udata[cp->stsc_e]*((1.0-r_stsc)*(1.0/data->c2n_slow) - (1.0/data->c2n_structural));
  cp->fstd = data->nitrogen2*udata[cp->msn_e]/(N_immobilization_demand +  data->nitrogen2*udata[cp->msn_e]);
#else
  N_immobilization_demand=0.0;
  cp->fstd = 1.0;
#endif

  mineralized_N_loss +=  N_immobilization_demand*cp->fstd;

  /*STRUCTURAL POOL*/
  /* compute associated decomposition of strucural C and lignin */ 
  structural_C_loss = A*Lc*K1*udata[cp->stsc_e]*cp->fstd;  
  structural_L_loss = A*Lc*K1*udata[cp->ssl_e]*cp->fstd;  


  /*fast pool*/
  /*NOTE: one of the fast pool losses in Century includes a soil type dependence thats not included here*/ 
  fast_C_loss =  A*K2*udata[cp->fsc_e];
  fast_N_loss =  A*K2*udata[cp->fsn_e];
  mineralized_N_input += fast_N_loss;

  /*slow pool*/
  slow_C_input = (1.0-r_stsc)*structural_C_loss;
  slow_C_loss =  A*K3*udata[cp->ssc_e];
  slow_N_loss = (1.0/data->c2n_slow)*slow_C_loss;
  mineralized_N_input += slow_N_loss;

  
  /*passive pool*/
  passive_C_input = 0.0;
  passive_C_loss =  A*K4*udata[cp->psc_e];
  passive_N_loss = (1.0/data->c2n_passive)*passive_C_loss;
  mineralized_N_input += passive_N_loss;

 /*C POOLS*/
  dudata[cp->fsc_e] = cp->fsc_in - fast_C_loss;
  dudata[cp->stsc_e] = cp->ssc_in - structural_C_loss;
  dudata[cp->ssc_e] = slow_C_input - slow_C_loss;
  dudata[cp->psc_e] = passive_C_input - passive_C_loss;
  dudata[cp->ssl_e] = cp->ssl_in - structural_L_loss;

  /*loss of C to atmosphere and leaching */
  cp->rh =   r_fsc*fast_C_loss + r_stsc*structural_C_loss + r_ssc*slow_C_loss + r_psc*passive_C_loss;

  /*NITROGEN POOLS*/
  dudata[cp->fsn_e] = cp->fsn_in - fast_N_loss;
  dudata[cp->msn_e] = mineralized_N_input - mineralized_N_loss;

#if DEBUG_INTEGRATOR
  printf("t= %f Dsdt: site = % p patch = %p \n",time,cs,cp);
  printf("t= %f Dsdt: A = %f \n",A);
  printf("t= %f Dsdt: rh = %f \n",time,cp->rh);

  printf("t= %f Dsdt: fsc_in = %f \n",time,cp->fsc_in);
  printf("t= %f Dsdt: ssc_in = %f \n",time,cp->ssc_in);
  printf("t= %f Dsdt: ssl_in = %f \n",time,cp->ssl_in);
  printf("t= %f Dsdt: fsc %f input %f loss %f  dfscdt= %f \n",time, udata[cp->fsc_e],cp->fsc_in,fast_C_loss,dudata[cp->fsc_e]);
  printf("t= %f Dsdt: stsc %f input %f loss %f dstscdt= %f \n",time,udata[cp->stsc_e],cp->ssc_in,structural_C_loss,dudata[cp->stsc_e]);
  printf("t= %f Dsdt: ssc %f input %f loss %f dsscdt= %f \n",time,udata[cp->ssc_e],slow_C_input,slow_C_loss,dudata[cp->ssc_e]);
  printf("t= %f Dsdt: psc %f input %f loss %f dpscdt= %f \n",time,udata[cp->psc_e],passive_C_input,passive_C_loss,dudata[cp->psc_e]);
  printf("t= %f Dsdt: fsn %f dfsndt= %f \n",time,udata[cp->fsn_e],dudata[cp->fsn_e]);
  printf("t= %f Dsdt: msn %f input %f loss %f plant_uptake %f dmsndt= %f \n",time,cp->mineralized_soil_N, mineralized_N_input, mineralized_N_loss, cp->total_plant_nitrogen_uptake,dudata[cp->msn_e]);
  printf("t= %f Dsdt: ssl %f input %f loss %f dssldt= %f \n",time,udata[cp->ssl_e],cp->ssl_in,structural_L_loss,dudata[cp->ssl_e]);
  printf("\n");
#endif

  return;

}
/*****************************************************************************/
float A_function(struct site **siteptr,struct patch **patchptr, unsigned int time_period,UserData data){
    
  /*The combined reduction in decomposition rate as a funciton of TEMP and MOIST*/
  /*Based on CENTURY Parton et al 1993 GBC 7(4):785-809*/
  /*and century code*/

  struct site *current_site;
  struct patch *current_patch;

  float soil_temp; /* temperature of the soil*/
  float rain;      /* precipitation*/
  float Td;        /* temperature rate multiplier*/
  float Wd;        /* moisture rate multiplier */
  float pet;       /* potential evapotranspiration*/

  /*coefficeints and terms used in temperaturex term*/
  float Topt,Tmax,t1,t2,tshl,tshr;

  current_site=*siteptr;
  current_patch=*patchptr;

  rain=current_site->sdata->precip[time_period];
  pet=current_site->sdata->pet[time_period];
  soil_temp=current_site->sdata->soil_temp[time_period];

  /*EFFECT OF TEMPERATURE*/
  /*from century code*/
  Tmax=45.0;
  Topt=35.0;
  tshr=0.2; tshl=2.63;
  t1=(Tmax-soil_temp)/(Tmax-Topt);
  t2=exp((tshr/tshl)*(1.-pow(t1,tshl)));
  Td=pow(t1,tshr)*t2;
  

  /*EFFECT OF MOISTURE*/
  /*Linn and Doran, 1984, Soil Sci. Amer. J. 48:1267-1272*/
  /*This differs from the Century Wd*/
  /*replaces PET and rainfall in Wd term*/

  if(current_patch->theta <= 0.3)
    Wd = 0.2;
  else if(current_patch->theta <= 0.6)
    Wd = current_patch->theta/0.6;
  else 
    Wd = 0.6/(1.2*current_patch->theta);

  /*printf("odeint: Td %f Wd %f A=%f\n",Td,Wd,Td*Wd);*/

  return(Td*Wd); /*the combined (multiplicative) effect of temp and water
		   on decomposition rates*/

}
/****************************************************************************/




